dofile("lua/inputscheme.lua")
dofile("lua/input.lua")
dofile("lua/table.save-0.94.lua")

Time = 0

ScreenQuad = -1
MainRendertarget = -1
PostProcessingRendertarget1 = -1
PostProcessingRendertarget2 = -1
StateObject = -1
MandelbrotXml = -1
MandelbrotShader = -1
ScreenfillXml = -1
ScreenfillShader = -1
RahmenXml = -1
RahmenShader = -1
Rahmen = -1
DrawRahmen = false

Globals = {}
ShaderConstants = {}

m_start = vec2(-3,1)
m_stop  = vec2(1,-1)
m_iterations = 300
m_recompute = true
m_time = 0
GuiActive = true

function Back()
  m_start = vec2(-3,1)
  m_stop = vec2(1,-1)
  m_recompute = true
  UpdateGui()
end

function MoveUp()
  local starty = m_start.y + (m_stop.y - m_start.y) / 20
  local stopy = m_stop.y + (m_stop.y - m_start.y) / 20
  m_start.y = starty
  m_stop.y = stopy
  m_recompute = true
  UpdateGui()
end

function MoveDown()
  local starty = m_start.y - (m_stop.y - m_start.y) / 20
  local stopy = m_stop.y - (m_stop.y - m_start.y) / 20
  m_start.y = starty
  m_stop.y = stopy
  m_recompute = true
  UpdateGui()
end

function MoveLeft()
  local startx = m_start.x - (m_stop.x - m_start.x) / 20
  local stopx = m_stop.x - (m_stop.x - m_start.x) / 20
  m_start.x = startx
  m_stop.x = stopx
  m_recompute = true
  UpdateGui()
end

function MoveRight()
  local startx = m_start.x + (m_stop.x - m_start.x) / 20
  local stopx = m_stop.x + (m_stop.x - m_start.x) / 20
  m_start.x = startx
  m_stop.x = stopx
  m_recompute = true
  UpdateGui()
end

function SwitchGui()
  if GuiActive then
    gui.ElementSetActive(InfoWindow,false)
    gui.SetActive(false)
    GuiActive = false
  else
    gui.ElementSetActive(InfoWindow,true)
    gui.SetActive(true)
    GuiActive = true
  end
end

function Init()
  collectgarbage("stop")
  Start = Zeitpunkt();

  if not renderer.TestMrtSupport() then
    gui.SetErrorMessageL("error_no_mrt_support")
    error("Hardware has no multiple rendertargets support")
  end

  if renderer.GetShaderModel() < 1 then
    gui.SetErrorMessageL("error_no_shader_support")
    error("Hardware has no shader support")
  end

  ips = InputScheme.Create()
  ips:SetBinaryEvent("back",19,Back)
  ips:SetBinaryEvent("quit",1,sys.Quit)
  ips:SetBinaryEvent("left",203,MoveLeft,50)
  ips:SetBinaryEvent("up",200,MoveUp,50)
  ips:SetBinaryEvent("right",205,MoveRight,50)
  ips:SetBinaryEvent("down",208,MoveDown,50)
  ips:SetBinaryEvent("gui",34,SwitchGui)
  ips:Use()

  Globals["WindowWidth"] = sys.GetGlobal("WindowWidth")
  Globals["WindowHeight"] = sys.GetGlobal("WindowHeight")

  ShaderConstants["m_start"] =renderer.GetShaderConstant("m_start")
  ShaderConstants["m_stop"] = renderer.GetShaderConstant("m_stop")
  ShaderConstants["m_iterations"] = renderer.GetShaderConstant("m_iterations")

  ScreenQuad = renderer.GetScreenQuad()

  MandelbrotXml = renderer.NewXmlShader("shader/mandelbrot.xml")
  renderer.XmlShaderUpload(MandelbrotXml)
  MandelbrotShader = renderer.XmlShaderGetShader(MandelbrotXml)

  ScreenfillXml = renderer.NewXmlShader("shader/test.xml")
  renderer.XmlShaderUpload(ScreenfillXml)
  ScreenfillShader = renderer.XmlShaderGetShader(ScreenfillXml)

  RahmenXml = renderer.NewXmlShader("shader/rahmen.xml")
  renderer.XmlShaderUpload(RahmenXml)
  RahmenShader = renderer.XmlShaderGetShader(RahmenXml)

  Farbverlauf = renderer.NewTexture2D("Farbverlauf")
  renderer.LoadPng(Farbverlauf,"gfx/verlauf.png","LINEAR","NO_LOCAL_DATA")

  --Create Main Rendertarget
  local ScreenWidth = Globals["WindowWidth"]
  local ScreenHeight = Globals["WindowHeight"]
  renderer.NewRendertargetBegin()
  renderer.RendertargetAddTarget("DEPTH_BUFFER","DEPTH")
  renderer.RendertargetAddTarget("COLOR_BUFFER","RGBA8") -- Normal
  MainRendertarget = renderer.NewRendertargetEnd(ScreenWidth,ScreenHeight)
  local error_string = renderer.RendertargetGetError(MainRendertarget)
  if error_string ~= nil then
    error(error_string)
  end
  --renderer.RendertargetPresetTargets(MainRendertarget,0,1)
  renderer.RendertargetSetClear(MainRendertarget,true,true,false)
  renderer.SetMainRendertarget(MainRendertarget)

  --Create Post Processing Rendertarget 1
  renderer.NewRendertargetBegin()
  renderer.RendertargetAddTarget("DEPTH_BUFFER","DEPTH")
  renderer.RendertargetAddTarget("COLOR_BUFFER","RGBA8")
  PostProcessingRendertarget1 = renderer.NewRendertargetEnd(ScreenWidth,ScreenHeight)
  error_string = renderer.RendertargetGetError(PostProcessingRendertarget1)
  if error_string ~= nil then
    error(error_string)
  end
  renderer.SetPostProcessingRendertarget1(PostProcessingRendertarget1)

  --Create Post Processing Rendertarget 2
  renderer.NewRendertargetBegin()
  renderer.RendertargetAddTarget("DEPTH_BUFFER","DEPTH")
  renderer.RendertargetAddTarget("COLOR_BUFFER","RGBA8")
  PostProcessingRendertarget2 = renderer.NewRendertargetEnd(ScreenWidth,ScreenHeight)
  error_string = renderer.RendertargetGetError(PostProcessingRendertarget2)
  if error_string ~= nil then
    error(error_string)
  end
  renderer.SetPostProcessingRendertarget2(PostProcessingRendertarget2)

  --Create Mandelbrot Target
  renderer.NewRendertargetBegin()
  renderer.RendertargetAddTarget("DEPTH_BUFFER","DEPTH")
  renderer.RendertargetAddTarget("COLOR_BUFFER","RGBA8")
  MandelbrotTarget = renderer.NewRendertargetEnd(ScreenWidth,ScreenHeight)
  error_string = renderer.RendertargetGetError(MandelbrotTarget)
  if error_string ~= nil then
    error(error_string)
  end
  MandelbrotTexture = renderer.RendertargetGetColorTexture(MandelbrotTarget,0)

  StateObject = renderer.NewStateObject()

  --Add Task Draw()
  DrawTask = tsk.New("LuaDraw","Draw")
  GarbageTask = tsk.New("Garbage Collector","CollectTheGarbage")
  OpenGLTask = tsk.GetTaskId("OpenGL Renderer")
  Profiler = tsk.GetTaskId("Profiler")
  tsk.AddDependencie(OpenGLTask,DrawTask)
  tsk.AddDependencie(Profiler,DrawTask)
  tsk.AddDependencie(Profiler,GarbageTask)

  tsk.AddDependencie(GarbageTask,DrawTask)

  renderer.RenderSliceSetActive(1,false)
  renderer.RenderSliceSetActive(2,false)
  --renderer.RenderSliceSetPostProcessing(2,true)
  --renderer.DrawFps(true)

  --Create Gui
  local Arial10 = gui.GetFont("Arial10")
  InfoWindow = gui.CreateWindow(-1,20,20,250,100,false,0,"parameters")
  InfoStart = gui.CreateText(InfoWindow,5,5,Arial10,"Start")
  InfoStop = gui.CreateText(InfoWindow,5,25,Arial10,"Stop")
  InfoIterations = gui.CreateText(InfoWindow,5,45,Arial10,"Iterations:")
  InfoIterationsInput = gui.CreateTextInput(InfoWindow,60,42,100,20,6,7,25,"300")
  gui.TextInputSetOnLooseFocus(InfoIterationsInput,"IterationsLooseFocus")
  UpdateGui()

  gui.SetActive(true)

  --Create VertexBuffer
  renderer.NewVertexBufferBegin()
  renderer.VertexBufferAddAttribute("POSITION",2)
  Rahmen = renderer.NewVertexBufferEnd("LINES","INDEX16")

  Stop = Zeitpunkt();
  LastDraw = Stop
  io.write("Init took "..(Stop-Start).." ms\n")
end

function IterationsLooseFocus()
  local temp = tonumber(gui.TextInputGetCaption(InfoIterationsInput))
  if temp == nil then
    gui.TextInputSetCaption(InfoIterationsInput,m_iterations)
  else
    m_iterations = temp
    m_recompute = true
  end
end

function UpdateGui()
  gui.TextSetCaption(InfoStart,string.format("Start X: %.8f Y: %.8f",m_start.x,m_start.y))
  gui.TextSetCaption(InfoStop,string.format("Stop X: %.8f Y: %.8f",m_stop.x,m_stop.y))
  gui.TextInputSetCaption(InfoIterationsInput,m_iterations)
end

function ThreadInit()
end

function DeInit()
  renderer.DeleteXmlShader(MandelbrotXml)
  renderer.DeleteXmlShader(ScreenfillXml)
  renderer.DeleteRendertarget(MainRendertarget)
  renderer.DeleteRendertarget(PostProcessingRendertarget1)
  renderer.DeleteRendertarget(PostProcessingRendertarget2)
end

function Draw()
  CurrentDraw = Zeitpunkt()
  TimeDiff = CurrentDraw - LastDraw
  Time = Time + (TimeDiff) / 1000
  RepeatEvent.Repeat(TimeDiff)

  -- Draw Scene
  if m_recompute then
    local start = Zeitpunkt()

    local RenderGroup = renderer.AddRenderGroup(0)
    renderer.RenderGroupSetRendertarget(RenderGroup,MandelbrotTarget)
    local RenderCall = renderer.AddRenderCall(RenderGroup)
    renderer.RenderCallSetVertexBuffer(RenderCall,ScreenQuad)
    renderer.RenderCallSetStateObject(RenderCall,StateObject)
    renderer.RenderCallSetShader(RenderCall,MandelbrotShader)
    renderer.RenderCallAddTexture(RenderCall,Farbverlauf,0)
    renderer.RenderCallOverrideVec2(RenderCall,ShaderConstants["m_start"],m_start)
    renderer.RenderCallOverrideVec2(RenderCall,ShaderConstants["m_stop"],m_stop)
    renderer.RenderCallOverrideFloat(RenderCall,ShaderConstants["m_iterations"],m_iterations)

    local stop = Zeitpunkt()
    m_time = stop - Start
    m_recompute = false
  end

  local RenderGroup2 = renderer.AddRenderGroup(0)
  local RenderCall2 = renderer.AddRenderCall(RenderGroup2)
  renderer.RenderCallSetVertexBuffer(RenderCall2,ScreenQuad)
  renderer.RenderCallSetStateObject(RenderCall2,StateObject)
  renderer.RenderCallSetShader(RenderCall2,ScreenfillShader)
  renderer.RenderCallAddTexture(RenderCall2,MandelbrotTexture,0)

  if DrawRahmen then
    RenderCall = renderer.AddRenderCall(RenderGroup2)
    renderer.RenderCallSetVertexBuffer(RenderCall,Rahmen)
    renderer.RenderCallSetShader(RenderCall,RahmenShader)
    renderer.RenderCallSetStateObject(RenderCall,StateObject)
  end

  LastDraw = CurrentDraw
end

function print_mat4(op)
  if not types.is_mat4(op) then
    error("Can not print a non mat4 calling print_mat4")
  end
  for y=0,3 do
    for x=0,3 do
      io.write(op[y*4+x].." ")
    end
    io.write("\n")
  end
end

function print_vec4(op)
  if not types.is_vec4(op) then
    error("Can not print a non vec4 calling print_vec4")
  end
  io.write(op.x..", "..op.y..", "..op.z..", "..op.w)
end

function CollectTheGarbage()
  collectgarbage("collect")
end
